#include <stdio.h>

// ein Enum ist eine Aufzählung von Namen die eine Ganzzahl entsprechen.
enum WochenTag{
  MONTAG, // = 0
  DIENSTAG, // = 1
  MITTWOCH, // ...
  DONNERSTAG,
  FREITAG,
  SAMSTAG,
  SONNTAG
};

// wir können auch explizit die Ganzzahl angeben:
enum Factor{
  KILO = 1000,
  MEGA = KILO*KILO, // wird zur Kompilierzeit ausgewertet
};

// eine Struktur um die Sonneneinstrahlung an einem bestimmten Wochentag anzugeben
struct SonnenEinstrahlung {
  enum WochenTag tag;
  float watts;
  enum Factor factor;
};

// ein Typedef erlaubt es Aliase für Typen festzulegen
// hier definieren wir WochenTag als enum WochenTag
typedef enum WochenTag WochenTag; // vorsicht, könnte zu Konfusion führen!

WochenTag aktuellerTag = DIENSTAG; // sonst hätten wir wieder "enum WochenTag aktuellerTag = DIENSTAG" schreiben müssen.

// oftmals wird daher an Strukturen ein Unterstrich angefügt, beim Typedef gibt es dann weniger Verwirrung.

// das geht natuerlich auch mit structs...

// hier definieren wir Einstrahlung als struct SonnenEinstrahlung.
typedef struct SonnenEinstrahlung Einstrahlung;

// Deklaration eines struct SonnenEinstrahlung se
Einstrahlung se;

int main(){

  // Definition einer Struktur mit enums
  se.tag = aktuellerTag;
  se.watts = 2.49f;
  se.factor = MEGA;

  printf("SonnenEinstrahlung: tag: %d, Watts: %f, factor: %i\n",se.tag, se.watts, se.factor);
 
  // typedef erlaubt es Typen zur Kompilierzeit überall festzulegen, z.b. FLIESKOMMA float oder double,
  // je nach gewünschter Genauigkeit
  
  // hier können wir entscheiden welche Rechnung durchgeführt wird:
  typedef int TYP;

  // hier berechnen wir die Länge eines Kreises
  for (int i=1; i < 10 ; i++){
    printf("Die Länge des Kreisumfanges mit Durchmesser %d ist %f\n",
	   i,
	   (double) ((TYP) (i * 3.141597 * 2.0))   );
  }

  {
  	// jetzt erhöhen wir die Genauigkeit auf Double:
  	typedef double TYP; // moeglich, da diese Definition von TYP in einem anderen Geltungsbereich ist

  	// selbe Rechnung:
  	for (int i=1; i < 10 ; i++){
    		printf("Die Länge des Kreisumfanges mit Durchmesser %d ist %f\n", i, (double) ((TYP) (i * 3.141597 * 2.0))   );
  	}
  }
}
